@@ -32,10 +32,10 @@
 #'   character vector} \item{text.mask}{the code containing comments, which are
 #'   masked in assignments or with the weird operator}
 #' @note Be sure to read the reference to know other limitations.
-#' @author Yihui Xie <\url{https://yihui.name}> with substantial contribution
-#'   from Yixuan Qiu <\url{http://yixuan.cos.name}>
+#' @author Yihui Xie <\url{https://yihui.org}> with substantial contribution
+#'   from Yixuan Qiu <\url{https://yixuan.blog}>
 #' @seealso \code{\link{parse}}, \code{\link{deparse}}
-#' @references \url{https://yihui.name/formatR} (an introduction to this
+#' @references \url{https://yihui.org/formatR} (an introduction to this
 #'   package, with examples and further notes)
 #' @import utils
 #' @export
@@ -54,10 +54,9 @@ tidy_source = function(
     if (source == 'clipboard' && Sys.info()['sysname'] == 'Darwin') {
       source = pipe('pbpaste')
     }
-  } else {
-    source = textConnection(text); on.exit(close(source), add = TRUE)
+    text = readLines(source, warn = FALSE)
   }
-  text = readLines(source, warn = FALSE)
+  enc = special_encoding(text)
   if (length(text) == 0L || all(grepl('^\\s*$', text))) {
     if (output) cat('\n', ...)
     return(list(text.tidy = text, text.mask = text))
@@ -76,7 +75,10 @@ tidy_source = function(
   # restore new lines in the beginning and end
   if (blank) text.tidy = c(rep('', n1), text.tidy, rep('', n2))
   if (output) cat(text.tidy, sep = '\n', ...)
-  invisible(list(text.tidy = text.tidy, text.mask = text.mask))
+  invisible(list(
+    text.tidy = restore_encoding(text.tidy, enc),
+    text.mask = restore_encoding(text.mask, enc)
+  ))
 }
 
 ## if you have variable names like this in your code, then you really beat me...
@@ -86,6 +88,7 @@ pat.comment = sprintf('invisible\\("\\%s|\\%s"\\)', begin.comment, end.comment)
 mat.comment = sprintf('invisible\\("\\%s([^"]*)\\%s"\\)', begin.comment, end.comment)
 inline.comment = ' %InLiNe_IdEnTiFiEr%[ ]*"([ ]*#[^"]*)"'
 blank.comment = sprintf('invisible("%s%s")', begin.comment, end.comment)
+blank.comment2 = sprintf('(\n)\\s+invisible\\("%s%s"\\)(\n|$)', begin.comment, end.comment)
 
 # wrapper around parse() and deparse()
 tidy_block = function(text, width = getOption('width'), arrow = FALSE) {
@@ -112,6 +115,8 @@ unmask_source = function(text.mask) {
     m = gregexpr(inline.comment, text.mask)
     regmatches(text.mask, m) = lapply(regmatches(text.mask, m), restore_bs)
   }
+  # remove white spaces on blank lines
+  text.mask = gsub(blank.comment2, '\\1\\2', text.mask)
   text.tidy = gsub(pat.comment, '', text.mask)
   # inline comments should be terminated by $ or \n
   text.tidy = gsub(paste(inline.comment, '(\n|$)', sep = ''), '  \\1\\2', text.tidy)
